/// <reference types="node" />
import Types, { TextTypeKnob, NumberTypeKnob, ColorTypeKnob, BooleanTypeKnob, ObjectTypeKnob, SelectTypeKnob, RadiosTypeKnob, ArrayTypeKnob, DateTypeKnob, ButtonTypeOnClickProp, FileTypeKnob, OptionsTypeKnob } from './components/types';
declare type Callback = () => any;
declare type KnobPlus<T extends keyof typeof Types, K> = K & {
    type: T;
    groupId?: string;
};
export declare type Knob = KnobPlus<'text', Pick<TextTypeKnob, 'value'>> | KnobPlus<'boolean', Pick<BooleanTypeKnob, 'value'>> | KnobPlus<'number', Pick<NumberTypeKnob, 'value' | 'range' | 'min' | 'max' | 'step'>> | KnobPlus<'color', Pick<ColorTypeKnob, 'value'>> | KnobPlus<'object', Pick<ObjectTypeKnob<any>, 'value'>> | KnobPlus<'select', Pick<SelectTypeKnob, 'value' | 'options'> & {
    selectV2: true;
}> | KnobPlus<'radios', Pick<RadiosTypeKnob, 'value' | 'options'>> | KnobPlus<'array', Pick<ArrayTypeKnob, 'value' | 'separator'>> | KnobPlus<'date', Pick<DateTypeKnob, 'value'>> | KnobPlus<'files', Pick<FileTypeKnob, 'value' | 'accept'>> | KnobPlus<'button', {
    value?: unknown;
    callback: ButtonTypeOnClickProp;
    hideLabel: true;
}> | KnobPlus<'options', Pick<OptionsTypeKnob<any>, 'options' | 'value' | 'optionsObj'>>;
export declare type KnobStoreKnob = Knob & {
    name: string;
    label: string;
    used?: boolean;
    defaultValue?: any;
    hideLabel?: boolean;
    callback?: () => any;
};
export default class KnobStore {
    store: Record<string, KnobStoreKnob>;
    callbacks: Callback[];
    timer: NodeJS.Timeout;
    has(key: string): boolean;
    set(key: string, value: KnobStoreKnob): void;
    get(key: string): KnobStoreKnob;
    getAll(): Record<string, KnobStoreKnob>;
    reset(): void;
    markAllUnused(): void;
    subscribe(cb: Callback): void;
    unsubscribe(cb: Callback): void;
}
export {};
